@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-#
 # Copyright 2014 - Mirantis, Inc.
 # Copyright 2014 - StackStorm, Inc.
 #
@@ -15,16 +13,21 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+from email import header
 from email.mime import text
+
 import json
 import requests
+import six
 import smtplib
+import time
 
 from mistral.actions import base
 from mistral import exceptions as exc
-from mistral.openstack.common import log as logging
+from mistral.utils import javascript
 from mistral.utils import ssh_utils
-
+from mistral.workflow import utils as wf_utils
+from oslo_log import log as logging
 
 LOG = logging.getLogger(__name__)
 
@@ -51,6 +54,50 @@ class EchoAction(base.Action):
         return 'Echo'
 
 
+class NoOpAction(base.Action):
+    """No-operation action.
+
+    This action does nothing. It can be mostly useful for testing and
+    debugging purposes.
+    """
+    def __init__(self):
+        pass
+
+    def run(self):
+        LOG.info('Running no-op action')
+
+        return None
+
+    def test(self):
+        return None
+
+
+class AsyncNoOpAction(NoOpAction):
+    """Asynchronous no-operation action."""
+    def is_sync(self):
+        return False
+
+
+class FailAction(base.Action):
+    """'Always fail' action.
+
+    This action just always throws an instance of ActionException.
+    This behavior is useful in a number of cases, especially if we need to
+    test a scenario where some of workflow tasks fail.
+    """
+
+    def __init__(self):
+        pass
+
+    def run(self):
+        LOG.info('Running fail action.')
+
+        raise exc.ActionException('Fail action expected exception.')
+
+    def test(self):
+        raise exc.ActionException('Fail action expected exception.')
+
+
 class HTTPAction(base.Action):
     """Constructs an HTTP action.
 
@@ -73,6 +120,8 @@ class HTTPAction(base.Action):
         redirect following is allowed.
     :param proxies: (optional) Dictionary mapping protocol to the URL of
         the proxy.
+    :param verify: (optional) if ``True``, the SSL cert will be verified.
+        A CA_BUNDLE path can also be provided.
     """
 
     def __init__(self,
@@ -85,13 +134,19 @@ class HTTPAction(base.Action):
                  auth=None,
                  timeout=None,
                  allow_redirects=None,
-                 proxies=None):
+                 proxies=None,
+                 verify=None):
 
         if auth and len(auth.split(':')) == 2:
             self.auth = (auth.split(':')[0], auth.split(':')[1])
         else:
             self.auth = auth
 
+        if isinstance(headers, dict):
+            for key, val in headers.items():
+                if isinstance(val, (six.integer_types, float)):
+                    headers[key] = str(val)
+
         self.url = url
         self.method = method
         self.params = params
@@ -101,12 +156,13 @@ class HTTPAction(base.Action):
         self.timeout = timeout
         self.allow_redirects = allow_redirects
         self.proxies = proxies
+        self.verify = verify
 
     def run(self):
         LOG.info("Running HTTP action "
                  "[url=%s, method=%s, params=%s, body=%s, headers=%s,"
                  " cookies=%s, auth=%s, timeout=%s, allow_redirects=%s,"
-                 " proxies=%s]" %
+                 " proxies=%s, verify=%s]" %
                  (self.url,
                   self.method,
                   self.params,
@@ -116,7 +172,8 @@ class HTTPAction(base.Action):
                   self.auth,
                   self.timeout,
                   self.allow_redirects,
-                  self.proxies))
+                  self.proxies,
+                  self.verify))
 
         try:
             resp = requests.request(
@@ -129,30 +186,43 @@ class HTTPAction(base.Action):
                 auth=self.auth,
                 timeout=self.timeout,
                 allow_redirects=self.allow_redirects,
-                proxies=self.proxies
+                proxies=self.proxies,
+                verify=self.verify
             )
         except Exception as e:
             raise exc.ActionException("Failed to send HTTP request: %s" % e)
 
-        LOG.info("HTTP action response:\n%s\n%s" %
-                 (resp.status_code, resp.content))
+        LOG.info(
+            "HTTP action response:\n%s\n%s" % (resp.status_code, resp.content)
+        )
 
-        # TODO(everyone): Not sure we need to have this check here in base HTTP
-        #                 action.
-        if resp.status_code not in range(200, 307):
-            raise exc.ActionException("Received error HTTP code: %s" %
-                                      resp.status_code)
-
-        # Construct all important resp data in readable structure.
-        headers = dict(resp.headers.items())
-        status = resp.status_code
+        # Represent important resp data as a dictionary.
         try:
             content = resp.json()
         except Exception as e:
             LOG.debug("HTTP action response is not json.")
             content = resp.content
 
-        return {'content': content, 'headers': headers, 'status': status}
+        _result = {
+            'content': content,
+            'status': resp.status_code,
+            'headers': dict(resp.headers.items()),
+            'url': resp.url,
+            'history': resp.history,
+            'encoding': resp.encoding,
+            'reason': resp.reason,
+            'cookies': dict(resp.cookies.items()),
+            'elapsed': resp.elapsed.total_seconds()
+        }
+
+        if resp.status_code not in range(200, 307):
+            return wf_utils.Result(error=_result)
+
+        return _result
+
+    def test(self):
+        # TODO(rakhmerov): Implement.
+        return None
 
 
 class MistralHTTPAction(HTTPAction):
@@ -167,12 +237,18 @@ class MistralHTTPAction(HTTPAction):
                  auth=None,
                  timeout=None,
                  allow_redirects=None,
-                 proxies=None):
+                 proxies=None,
+                 verify=None):
+
+        actx = action_context
+
         headers = headers or {}
         headers.update({
-            'Mistral-Workflow-Name': action_context.get('workflow_name'),
-            'Mistral-Execution-Id': action_context.get('execution_id'),
-            'Mistral-Task-Id': action_context.get('task_id'),
+            'Mistral-Workflow-Name': actx.get('workflow_name'),
+            'Mistral-Workflow-Execution-Id': actx.get('workflow_execution_id'),
+            'Mistral-Task-Id': actx.get('task_id'),
+            'Mistral-Action-Execution-Id': actx.get('action_execution_id'),
+            'Mistral-Callback-URL': actx.get('callback_url'),
         })
 
         super(MistralHTTPAction, self).__init__(
@@ -186,26 +262,30 @@ class MistralHTTPAction(HTTPAction):
             timeout,
             allow_redirects,
             proxies,
+            verify,
         )
 
     def is_sync(self):
         return False
 
+    def test(self):
+        return None
+
 
 class SendEmailAction(base.Action):
-    def __init__(self, params, settings):
+    def __init__(self, from_addr, to_addrs, smtp_server,
+                 smtp_password=None, subject=None, body=None):
         # TODO(dzimine): validate parameters
 
         # Task invocation parameters.
-        self.to = ', '.join(params['to'])
-        self.subject = params['subject']
-        self.body = params['body']
+        self.to = to_addrs
+        self.subject = subject or "<No subject>"
+        self.body = body or "<No body>"
 
         # Action provider settings.
-        self.smtp_server = settings['smtp_server']
-        self.sender = settings['from']
-        self.password = settings['password'] \
-            if 'password' in settings else None
+        self.smtp_server = smtp_server
+        self.sender = from_addr
+        self.password = smtp_password
 
     def run(self):
         LOG.info("Sending email message "
@@ -213,11 +293,10 @@ class SendEmailAction(base.Action):
                  (self.sender, self.to, self.subject,
                   self.smtp_server, self.body[:128]))
 
-        # TODO(dzimine): handle utf-8, http://stackoverflow.com/a/14506784
-        message = text.MIMEText(self.body)
-        message['Subject'] = self.subject
+        message = text.MIMEText(self.body, _charset='utf-8')
+        message['Subject'] = header.Header(self.subject, 'utf-8')
         message['From'] = self.sender
-        message['To'] = self.to
+        message['To'] = ', '.join(self.to)
 
         try:
             s = smtplib.SMTP(self.smtp_server)
@@ -246,11 +325,32 @@ class SendEmailAction(base.Action):
 
 
 class SSHAction(base.Action):
-    def __init__(self, cmd, host, username, password):
+    """Runs Secure Shell (SSH) command on provided single or multiple hosts.
+
+    It is allowed to provide either a single host or a list of hosts in
+    action parameter 'host'. In case of a single host the action result
+    will be a single value, otherwise a list of results provided in the
+    same order as provided hosts.
+    """
+    @property
+    def _execute_cmd_method(self):
+        return ssh_utils.execute_command
+
+    def __init__(self, cmd, host, username,
+                 password=None, private_key_filename=None):
         self.cmd = cmd
         self.host = host
         self.username = username
         self.password = password
+        self.private_key_filename = private_key_filename
+
+        self.params = {
+            'cmd': self.cmd,
+            'host': self.host,
+            'username': self.username,
+            'password': self.password,
+            'private_key_filename': self.private_key_filename
+        }
 
     def run(self):
         def raise_exc(parent_exc=None):
@@ -261,13 +361,130 @@ class SSHAction(base.Action):
             raise exc.ActionException(message)
 
         try:
-            status_code, result = ssh_utils.execute_command(self.cmd,
-                                                            self.host,
-                                                            self.username,
-                                                            self.password)
-            if status_code > 0:
-                return raise_exc()
+            results = []
+
+            if not isinstance(self.host, list):
+                self.host = [self.host]
+
+            for host_name in self.host:
+                self.params['host'] = host_name
+
+                status_code, result = self._execute_cmd_method(**self.params)
+
+                if status_code > 0:
+                    return raise_exc()
+                else:
+                    results.append(result)
+
+            if len(results) > 1:
+                return results
 
             return result
         except Exception as e:
             return raise_exc(parent_exc=e)
+
+    def test(self):
+        # TODO(rakhmerov): Implement.
+        return None
+
+
+class SSHProxiedAction(SSHAction):
+    @property
+    def _execute_cmd_method(self):
+        return ssh_utils.execute_command_via_gateway
+
+    def __init__(self, cmd, host, username, private_key_filename,
+                 gateway_host, gateway_username=None,
+                 password=None, proxy_command=None):
+        super(SSHProxiedAction, self).__init__(
+            cmd,
+            host,
+            username,
+            password,
+            private_key_filename
+        )
+
+        self.gateway_host = gateway_host
+        self.gateway_username = gateway_username
+
+        self.params.update(
+            {
+                'gateway_host': gateway_host,
+                'gateway_username': gateway_username,
+                'proxy_command': proxy_command
+            }
+        )
+
+
+class JavaScriptAction(base.Action):
+    """Evaluates given JavaScript.
+
+    """
+    def __init__(self, script, context=None):
+        self.script = script
+        self.context = context
+
+    def run(self):
+        try:
+            script = """function f() {
+                %s
+            }
+            f()
+            """ % self.script
+            return javascript.evaluate(script, self.context)
+        except Exception as e:
+            raise exc.ActionException("JavaScriptAction failed: %s" % str(e))
+
+    def test(self):
+        return self.script
+
+
+class SleepAction(base.Action):
+    """Sleep action.
+
+    This action sleeps for given amount of seconds. It can be mostly useful
+    for testing and debugging purposes.
+    """
+    def __init__(self, seconds=1):
+        try:
+            self._seconds = int(seconds)
+            self._seconds = 0 if self._seconds < 0 else self._seconds
+        except ValueError:
+            self._seconds = 0
+
+    def run(self):
+        LOG.info('Running sleep action [seconds=%s]' % self._seconds)
+
+        time.sleep(self._seconds)
+
+        return None
+
+    def test(self):
+        time.sleep(1)
+
+        return None
+
+
+class TestDictAction(base.Action):
+    """Generates test dict."""
+
+    def __init__(self, size=0, key_prefix='', val=''):
+        self.size = size
+        self.key_prefix = key_prefix
+        self.val = val
+
+    def run(self):
+        LOG.info(
+            'Running test_dict action [size=%s, key_prefix=%s, val=%s]' %
+            (self.size, self.key_prefix, self.val)
+        )
+
+        res = {}
+
+        for i in range(self.size):
+            res['%s%s' % (self.key_prefix, i)] = self.val
+
+        return res
+
+    def test(self):
+        return {}
