@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
-#
 # Copyright 2013 - Mirantis, Inc.
 # Copyright 2015 - Huawei Technologies Co. Ltd
+# Copyright 2016 - Brocade Communications Systems, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -16,20 +15,22 @@
 #    limitations under the License.
 
 import contextlib
+import functools
 import json
 import logging
 import os
 from os import path
 import shutil
-import six
 import socket
+import sys
 import tempfile
 import threading
-import uuid
 
 import eventlet
 from eventlet import corolocal
 from oslo_concurrency import processutils
+from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import pkg_resources as pkg
 import random
 
@@ -39,10 +40,16 @@ from mistral import version
 
 # Thread local storage.
 _th_loc_storage = threading.local()
+ACTION_TASK_TYPE = 'ACTION'
+WORKFLOW_TASK_TYPE = 'WORKFLOW'
 
 
 def generate_unicode_uuid():
-    return six.text_type(str(uuid.uuid4()))
+    return uuidutils.generate_uuid()
+
+
+def is_valid_uuid(uuid_string):
+    return uuidutils.is_uuid_like(uuid_string)
 
 
 def _get_greenlet_local_storage():
@@ -73,7 +80,7 @@ def get_thread_local(var_name):
 
 
 def set_thread_local(var_name, val):
-    if not val and has_thread_local(var_name):
+    if val is None and has_thread_local(var_name):
         gl_storage = _get_greenlet_local_storage()
 
         # Delete variable from greenlet local storage.
@@ -84,7 +91,7 @@ def set_thread_local(var_name, val):
         if gl_storage and len(gl_storage) == 0:
             del _th_loc_storage.greenlet_locals[corolocal.get_ident()]
 
-    if val:
+    if val is not None:
         gl_storage = _get_greenlet_local_storage()
         if not gl_storage:
             gl_storage = _th_loc_storage.greenlet_locals[
@@ -95,13 +102,15 @@ def set_thread_local(var_name, val):
 
 def log_exec(logger, level=logging.DEBUG):
     """Decorator for logging function execution.
+
         By default, target function execution is logged with DEBUG level.
     """
 
     def _decorator(func):
+        @functools.wraps(func)
         def _logged(*args, **kw):
             params_repr = ("[args=%s, kw=%s]" % (str(args), str(kw))
-                           if len(args) > 0 or len(kw) > 0 else "")
+                           if args or kw else "")
 
             func_repr = ("Called method [name=%s, doc='%s', params=%s]" %
                          (func.__name__, func.__doc__, params_repr))
@@ -132,7 +141,7 @@ def merge_dicts(left, right, overwrite=True):
     if right is None:
         return left
 
-    for k, v in six.iteritems(right):
+    for k, v in right.items():
         if k not in left:
             left[k] = v
         else:
@@ -146,6 +155,25 @@ def merge_dicts(left, right, overwrite=True):
     return left
 
 
+def update_dict(left, right):
+    """Updates left dict with content from right dict
+
+    :param left: Left dict.
+    :param right: Right dict.
+    :return: the updated left dictionary.
+    """
+
+    if left is None:
+        return right
+
+    if right is None:
+        return left
+
+    left.update(right)
+
+    return left
+
+
 def get_file_list(directory):
     base_path = pkg.resource_filename(
         version.version_info.package,
@@ -156,16 +184,115 @@ def get_file_list(directory):
             if path.isfile(path.join(base_path, f))]
 
 
+def cut_dict(d, length=100):
+    if not isinstance(d, dict):
+        raise ValueError("A dictionary is expected, got: %s" % type(d))
+
+    res = "{"
+
+    idx = 0
+
+    for key, value in d.items():
+        k = str(key)
+        v = str(value)
+
+        # Processing key.
+        new_len = len(res) + len(k)
+
+        is_str = isinstance(key, str)
+
+        if is_str:
+            new_len += 2
+
+        if new_len >= length:
+            res += "'%s..." % k[:length - new_len] if is_str else "%s..." % k
+
+            break
+        else:
+            res += "'%s'" % k if is_str else k
+            res += ": "
+
+        # Processing value.
+        new_len = len(res) + len(v)
+
+        is_str = isinstance(value, str)
+
+        if is_str:
+            new_len += 2
+
+        if new_len >= length:
+            res += "'%s..." % v[:length - new_len] if is_str else "%s..." % v
+
+            break
+        else:
+            res += "'%s'" % v if is_str else v
+            res += ', ' if idx < len(d) - 1 else '}'
+
+        if len(res) >= length:
+            res += '...'
+
+            break
+
+        idx += 1
+
+    return res
+
+
+def cut_list(l, length=100):
+    if not isinstance(l, list):
+        raise ValueError("A list is expected, got: %s" % type(l))
+
+    res = '['
+
+    for idx, item in enumerate(l):
+        s = str(item)
+
+        new_len = len(res) + len(s)
+
+        is_str = isinstance(item, str)
+
+        if is_str:
+            new_len += 2
+
+        if new_len >= length:
+            res += "'%s..." % s[:length - new_len] if is_str else "%s..." % s
+
+            break
+        else:
+            res += "'%s'" % s if is_str else s
+            res += ', ' if idx < len(l) - 1 else ']'
+
+    return res
+
+
+def cut_string(s, length=100):
+    if len(s) > length:
+        return "%s..." % s[:length]
+
+    return s
+
+
 def cut(data, length=100):
     if not data:
         return data
 
-    string = str(data)
+    if isinstance(data, list):
+        return cut_list(data, length=length)
 
-    if len(string) > length:
-        return "%s..." % string[:length]
-    else:
-        return string
+    if isinstance(data, dict):
+        return cut_dict(data, length=length)
+
+    return cut_string(str(data), length=length)
+
+
+def cut_by_kb(data, kilobytes):
+    if kilobytes <= 0:
+        return cut(data)
+
+    bytes_per_char = sys.getsizeof('s') - sys.getsizeof('')
+    length = int(kilobytes * 1024 / bytes_per_char)
+
+    return cut(data, length)
 
 
 def iter_subclasses(cls, _seen=None):
@@ -178,7 +305,7 @@ def iter_subclasses(cls, _seen=None):
 
     try:
         subs = cls.__subclasses__()
-    except TypeError:   # fails only when cls is type
+    except TypeError:  # fails only when cls is type
         subs = cls.__subclasses__(cls)
 
     for sub in subs:
@@ -267,6 +394,7 @@ def tempdir(**kwargs):
 
     if 'dir' not in argdict:
         argdict['dir'] = '/tmp/'
+
     tmpdir = tempfile.mkdtemp(**argdict)
 
     try:
@@ -281,8 +409,19 @@ def tempdir(**kwargs):
             )
 
 
+def save_text_to(text, file_path, overwrite=False):
+    if os.path.exists(file_path) and not overwrite:
+        raise exc.DataAccessException(
+            "Cannot save data to file. File %s already exists."
+        )
+
+    with open(file_path, 'w') as f:
+        f.write(text)
+
+
 def generate_key_pair(key_length=2048):
     """Create RSA key pair with specified number of bits in key.
+
     Returns tuple of private and public keys.
     """
     with tempdir() as tmpdir:
@@ -295,15 +434,20 @@ def generate_key_pair(key_length=2048):
             '-f', keyfile,  # filename of the key file
             '-C', 'Generated-by-Mistral'  # key comment
         ]
+
         if key_length is not None:
             args.extend(['-b', key_length])
+
         processutils.execute(*args)
+
         if not os.path.exists(keyfile):
             raise exc.DataAccessException(
                 "Private key file hasn't been created"
             )
+
         private_key = open(keyfile).read()
         public_key_path = keyfile + '.pub'
+
         if not os.path.exists(public_key_path):
             raise exc.DataAccessException(
                 "Public key file hasn't been created"
@@ -311,3 +455,10 @@ def generate_key_pair(key_length=2048):
         public_key = open(public_key_path).read()
 
         return private_key, public_key
+
+
+def utc_now_sec():
+    """Returns current time and drops microseconds."""
+
+    d = timeutils.utcnow()
+    return d.replace(microsecond=0)
